//------------------------------------------------------------------------------
// Theta-LittlePlanet  version 0.1  by E.Kako @kako.com (c)2013
//------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "opencv2/opencv.hpp"

//------------------------------------------------------------------------------
// Main program
//------------------------------------------------------------------------------
int main(int argc,char **argv)
{
	IplImage *image,*image_,*image1;
	int wd,ht;
	char fname[1024];
	
	printf("<<< theta-littleplanet  version 0.1  by E.kako @kako.com >>>\n");

	if (argc!=2) {
		printf("Usage: theta-littleplanet filename\n"); return 0;
	}
	
	strcpy(fname,argv[1]);
	
	// load imagefile
	image = cvLoadImage(fname,CV_LOAD_IMAGE_ANYCOLOR);
	if (image == NULL) {
		printf("Error: cannot load image file.\n"); return 1;
	}
	
	// image size
	wd = image->width; ht = image->height;
	
	// image memory
	image1 = cvCreateImage(cvSize(wd,ht),IPL_DEPTH_8U, 3);
	
	// make little planet
	{
		double th,ph;
		double th0,ph0;
		CvScalar s0;
		double x0,y0;
		double x1,y1,z1;
		double x2,y2;
		double x,y,z;
		double xr,yr,zr;
		double r;
		double rot;
		double pi = 3.14159265358979323846;
		double range = 150;
		double delta = 0.1;
		int tic = 0;
		
		// fill black
		cvSet(image1, cvScalarAll(0), 0 );
		
		for (th0=-range;th0<+range;th0+=delta) {
			tic++;
			if (tic%200==0) { printf("."); fflush(stdout); }
			for (ph0=-range;ph0<+range;ph0+=delta) {
				
				if ((th0<range*0.6)&&(th0>-range*0.6)&&(ph0<range*0.6)&&(ph0>-range*0.6)) {
					delta = 0.1;
				} else if ((th0<range*0.7)&&(th0>-range*0.7)&&(ph0<range*0.7)&&(ph0>-range*0.7)) {
					delta = 0.075;
				} else if ((th0<range*0.9997)&&(th0>-range*0.9997)&&(ph0<range*0.9997)&&(ph0>-range*0.9997)) {
					delta = 0.05;
				} else if ((th0<range*0.9999)&&(th0>-range*0.9999)&&(ph0<range*0.9999)&&(ph0>-range*0.9999)) {
					delta = 0.025;
				} else {
					delta = 0.0125;
				}
				
				// th0,ph0 --> x,y,z
				x = 1.0 * cos(ph0/180*pi)*cos(th0/180*pi);
				y = 1.0 * cos(ph0/180*pi)*sin(th0/180*pi);
				z = 1.0 * sin(ph0/180*pi);
				
				// rotate 90 degree
				rot=-90;
				yr = y;
				zr = z*cos(rot/180*pi) - x*sin(rot/180*pi);
				xr = z*sin(rot/180*pi) + x*cos(rot/180*pi);
				if (zr>1) { zr=1; }
				
				// rotated xr,yr,zr --> th,pi
				ph=asin(zr)*180/pi;
				th=atan2(yr,xr)*180/pi;
				if (ph>90) { ph=90; } else if (ph<-90) { ph=-90; }
				if (th>180) { th-=360; } else if (th<-180) { th+=360; }
				
				// th,ph --> x0,y0 in original jpeg image
				x0 = th/180 * (wd/2) + (wd/2);
				y0 = ph/90 * (ht/2) + (ht/2);
				if ((x0<0)||(x0>=wd)) { continue; }
				if ((y0<0)||(y0>=ht)) { continue; }
				
				// sphere
				r= ht;
				x1 = r * cos(ph0/180*pi)*sin(th0/180*pi);
				y1 = r * cos(ph0/180*pi)*cos(th0/180*pi);
				z1 = r * sin(ph0/180*pi);
				
				// 3D --> 2D projection
				x2 = x1 * r*0.3 / (r+y1);
				y2 = z1 * r*0.3 / (r+y1);
				if ((x2*x2+y2*y2)>(r*r*0.5*0.5)) { continue; }
				if ((x2>ht/2)||(x2<-ht/2)) { continue; }
				x2 += wd/2;
				y2 += ht/2;
				if ((x2>=wd)||(x2<0)) { continue; }
				if ((y2>=ht)||(y2<0)) { continue; }
				
				s0= cvGet2D(image1,y2,x2);
				if ((s0.val[0]==0)&&(s0.val[1]==0)&&(s0.val[2]==0)&&(s0.val[3]==0)) {
					cvSet2D(image1,y2,x2,cvGet2D(image,y0,x0));
				}
			}
		}
	}
	printf("\n");
	
	// save image file
	strcat(fname,".littleplanet.jpg");
	cvSaveImage(fname, image1);
	printf("generated image(%s).\n",fname);
	
	// release memory
	cvReleaseImage(&image);
	cvReleaseImage(&image1);
	
	return 0;
}
